FUNCTION MemoPagr
*-------------------------------------------------------------------------------
*-- Programmer..: Martin Leon (HMAN - ATBBS/Borland BBS)
*-- Date........: 10/28/91
*-- Notes.......: Used to display a memo on screen, allowing user to scroll
*--               memo at will.
*-- Written for.: dBASE IV, 1.1
*-- Rev. History: 
*-- Calls.......: None
*-- Called by...: Any
*-- Usage.......: ?MemoPagr(<memoname>,<ULRow>,<ULCol>,<BRRow>,<BRCol>)
*-- Example.....: ?MemoPagr(MoreData,10,20,20,65)
*-- Returns.....: .F.
*-- Parameters..: cMemo   = name of memo field
*--               nULRow  = upper left row position
*--               nULCol  = upper left column position
*--               nBRRow  = bottom right row position
*--               nBRCol  = bottom right column position
*-------------------------------------------------------------------------------
   
   PARAMETER cMemo, nULRow, nULCol, nBRRow, nBRCol

   *-- set environment
   set memowidth to nBRCol - nULCol - 1
   cCursor = set( "CURSOR" )
   set cursor off
   
   *-- define a few keys
   nEsc = 27
   nPgDn = 3
   nPgUp = 18
   nUp = 5
   nDn = 24
   
   *-- determine size of window
   nNumLines = memlines(&cMemo)
   nLines = nBRRow - nULRow - 1
   *-- save the screen, so we can restore it
   save screen to sTmp
   @ nULRow+1, nULCol+1 clear to nBRRow+1, nBRCol+1
   @ nULRow+1, nULCol+1 fill to nBRRow+1, nBRCol+1 color B/N
   @ nULRow+1, nULCol+1 fill to nBRRow-1, nBRCol-1 color RG+/B
   @ nULRow, nULCol to nBRRow, nBRCol double color RG+/B
   
   *-- deal with a blank memo ...
   if nNumLines = 0
      @ nULRow + 1, nULCol + 1 SAY ;
         "Blank Memo.  Press any key to continue..." color RG+/B
      nKey = inkey(0)
      *-- reset the whole thing
      restore screen from sTmp
      release screen sTmp
      set cursor &cCursor
      RETURN .F.
   endif
   
   nAtLine = 1
   nAtRow = 1
   do while nAtLine <= nNumLines
      *-- Show one window full
      do while nAtRow <= nLines .and. nAtLine <= nNumLines
         @ nULRow+nAtRow, nULCol + 1 say ;
            mline( &cMemo, nAtLine ) color RG+/B
         nAtLine = nAtLine + 1
         nAtRow = nAtRow + 1
      enddo
   
      *-- If at last line of memo...
      if nAtLine > nNumLines
         *-- If memo is shorter than one page, put box character in
         *-- bottom left corner of box, otherwise, put an up arrow
         *-- symbol there.
         @ nBRRow - 1, nBRCol SAY ;
         iif( nNumLines <= nLines, chr(186), chr(24)) color W+/B
         do while .T.
            nKey = inkey(0)
            *-- If memo is shorter than one page, only allow Esc key
            if nNumLines <= nLines
               if nKey = nEsc
                  exit
               endif
            *-- Otherwise, allow Esc or PgUp keys
            else
               if nKey = nEsc .or. nKey = nPgUp .or. nKey = nUp
                  exit
               endif
            endif
            ?? chr(7)
         enddo
         if nKey = nEsc
            restore screen from sTmp
            release screen sTmp
            set cursor &cCursor
            RETURN .F.
         endif
         @ nULRow+1, nULCol+1 clear to nBRRow-1, nBRCol-1
         @ nULRow+1, nULCol+1 fill to nBRRow-1, nBRCol-1 ;
            color RG+/B
         nAtLine = nAtLine -  nAtRow - nLines + 1
         nAtLine = iif( nAtLine < 1, 1, nAtLine )
         nAtRow = 1
         loop
      endif
   
      *-- Not at end of memo yet...
      *-- If on first page, show down arrow only, otherwise show
      *-- up/down arrow on border of box.
      @ nBRRow - 1, nBRCol say ;
         iif( nAtLine - nLines = 1, chr(25), chr(18)) color W+/B
      do while .T.
         nKey = inkey(0)
         *-- If this is the first page of the memo on screen...
         if nAtLine - nLines = 1
            *-- Only honor PgDn, up cursor, and Esc keys
            if nKey = nPgDn .or. nKey = nDn .or. nKey = nEsc
               exit
            endif
         *-- otherwise honor PgUp and up cursor as well key as well
         else 
            if nKey = nPgUp .or. nKey = nUp .or. nKey = nPgDn .or. ;
                   nKey = nDn .or. nKey = nEsc
               exit
            endif
         endif
         ?? chr(7)
      enddo
      do case
         case nKey = nEsc
            restore screen from sTmp
            release screen sTmp
            set cursor &cCursor
            RETURN .F.
         case nKey = nPgUp .or. nKey = nUp
            @ nULRow+1, nULCol+1 clear to nBRRow-1, nBRCol-1
            @ nULRow+1, nULCol+1 fill to nBRRow-1, nBRCol-1 ;
               color RG+/B
            nAtLine = (nAtLine - (2 * nLines))
            nAtLine = IIF( nAtLine < 1, 1, nAtLine )
            nAtRow = 1
            loop
         case nKey = nPgDn .or. nKey = nDn
            @ nULRow+1, nULCol+1 clear to nBRRow-1, nBRCol-1
            @ nULRow+1, nULCol+1 fill to nBRRow-1, nBRCol-1 ;
               color RG+/B
            nAtRow = 1
            loop
      endcase
   enddo

RETURN .F.
*-- EoF: MemoPagr
